#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file ModuleManager.py
# @brief Loadable modules manager class
# @date $Date: 2007/08/24$
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2006-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.



import sys,os
import glob

import OpenRTM_aist


CONFIG_EXT    = "manager.modules.config_ext"
CONFIG_PATH   = "manager.modules.config_path"
DETECT_MOD    = "manager.modules.detect_loadable"
MOD_LOADPTH   = "manager.modules.load_path"
INITFUNC_SFX  = "manager.modules.init_func_suffix"
INITFUNC_PFX  = "manager.modules.init_func_prefix"
ALLOW_ABSPATH = "manager.modules.abs_path_allowed"
ALLOW_URL     = "manager.modules.download_allowed"
MOD_DWNDIR    = "manager.modules.download_dir"
MOD_DELMOD    = "manager.modules.download_cleanup"
MOD_PRELOAD   = "manager.modules.preload"



##
# @if jp
#
# @brief ⥸塼ޥ͡㥯饹
# @class ModuleManager
#
# ⥸塼Υɡɤʤɤ륯饹
#
# @since 0.4.0
#
# @else
#
# @biref ModuleManager class
#
# @endif
class ModuleManager:
  """
  """



  ##
  # @if jp
  #
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  # ꤵ줿 Property ֥ξ˽¹Ԥ롣
  #
  # @param self
  # @param prop ѥץѥƥ
  #
  # @else
  #
  # @brief constructor
  #
  # @endif
  def __init__(self, prop):
    self._properties = prop

    self._configPath = prop.getProperty(CONFIG_PATH).split(",")
    for i in range(len(self._configPath)):
      tmp = [self._configPath[i]]
      OpenRTM_aist.eraseHeadBlank(tmp)
      self._configPath[i] = tmp[0]
    self._loadPath = prop.getProperty(MOD_LOADPTH,"./").split(",")
    for i in range(len(self._loadPath)):
      tmp = [self._loadPath[i]]
      OpenRTM_aist.eraseHeadBlank(tmp)
      self._loadPath[i] = tmp[0]

    self._absoluteAllowed = OpenRTM_aist.toBool(prop.getProperty(ALLOW_ABSPATH),
                                                "yes", "no", False)

    self._downloadAllowed = OpenRTM_aist.toBool(prop.getProperty(ALLOW_URL),
                                                "yes", "no", False)

    self._initFuncSuffix = prop.getProperty(INITFUNC_SFX)
    self._initFuncPrefix = prop.getProperty(INITFUNC_PFX)
    self._modules = OpenRTM_aist.ObjectManager(self.DLLPred)
    self._rtcout = None
    self._mgr = OpenRTM_aist.Manager.instance()
    if not self._rtcout:
      self._rtcout = self._mgr.getLogbuf("ModuleManager")
    self._modprofs = []
    self._loadfailmods = []

  ##
  # @if jp
  #
  # @brief ǥȥ饯(̤)
  #
  # @param self
  #
  # @else
  #
  # @brief destructor
  #
  # @endif
  def __del__(self):
    self.unloadAll()


  ##
  # @if jp
  # @class Error
  # @brief ե롦ץ㳰饹
  # @else
  #
  # @endif
  class Error(Exception):
    def __init__(self, reason_):
      self.reason = reason_



  ##
  # @if jp
  # @class NotFound
  # @brief ̤⥸塼㳰饹
  # @else
  #
  # @endif
  class NotFound(Exception):
    def __init__(self, name_):
      self.name = name_



  ##
  # @if jp
  # @class FileNotFound
  # @brief ե㳰饹
  # @else
  #
  # @endif
  class FileNotFound(NotFound):
    def __init__(self, name_):
      ModuleManager.NotFound.__init__(self, name_)



  ##
  # @if jp
  # @class ModuleNotFound
  # @brief ⥸塼㳰饹
  # @else
  #
  # @endif
  class ModuleNotFound(NotFound):
    def __init__(self, name_):
      ModuleManager.NotFound.__init__(self, name_)



  ##
  # @if jp
  # @class SymbolNotFound
  # @brief ꥷܥ㳰饹
  # @else
  #
  # @endif
  class SymbolNotFound(NotFound):
    def __init__(self, name_):
      ModuleManager.NotFound.__init__(self, name_)



  ##
  # @if jp
  # @class NotAllowedOperation
  # @brief ػ߻㳰饹
  # @else
  #
  # @endif
  class NotAllowedOperation(Error):
    def __init__(self, reason_):
      ModuleManager.Error.__init__(self, reason_)



  ##
  # @if jp
  # @class InvalidArguments
  # @brief 㳰饹
  # @else
  #
  # @endif
  class InvalidArguments(Error):
    def __init__(self, reason_):
      ModuleManager.Error.__init__(self, reason_)



  ##
  # @if jp
  # @class InvalidOperation
  # @brief 㳰饹
  # @else
  #
  # @endif
  class InvalidOperation(Error):
    def __init__(self, reason_):
      ModuleManager.Error.__init__(self, reason_)



  ##
  # @if jp
  #
  # @brief ⥸塼Υɡ
  #
  # ꤷեͭ饤֥ȤƥɤȤȤˡ
  # ꤷѥڥ졼¹Ԥ롣
  # 
  # @param self
  # @param file_name оݥ⥸塼̾ (.pyե̾)
  # @param init_func ѥڥ졼(ǥե:None)
  #
  # @return ꤷоݥ⥸塼̾
  #
  # @else
  #
  # @brief Load module
  #
  #
  # @endif
  # std::string ModuleManager::load(const std::string& file_name,
  #                                 const std::string& init_func)
  def load(self, file_name, init_func=None):
    

    self._rtcout.RTC_TRACE("load(fname = %s)", file_name)
    if file_name == "":
      raise ModuleManager.InvalidArguments("Invalid file name.")

    if OpenRTM_aist.isURL(file_name):
      if not self._downloadAllowed:
        raise ModuleManager.NotAllowedOperation("Downloading module is not allowed.")
      else:
        raise ModuleManager.NotFound("Not implemented.")
    
    import_name = os.path.split(file_name)[-1]
    pathChanged=False
    file_path = None
    
    if OpenRTM_aist.isAbsolutePath(file_name):
      if not self._absoluteAllowed:
        raise ModuleManager.NotAllowedOperation("Absolute path is not allowed")
      else:
        splitted_name = os.path.split(file_name)
        save_path = sys.path[:]
        sys.path.append(splitted_name[0])
        
        pathChanged = True
        import_name = splitted_name[-1]
        file_path = file_name

    else:
      file_path = self.findFile(file_name, self._loadPath)
      if not file_path:
        raise ModuleManager.FileNotFound(file_name)
  
    if not self.fileExist(file_path):
      raise ModuleManager.FileNotFound(file_name)

    
    with open(str(file_path)) as f:
      if init_func is not None:
        if f.read().find(init_func) == -1:
          raise ModuleManager.FileNotFound(file_name)
          

    if not pathChanged:
      splitted_name = os.path.split(file_path)
      sys.path.append(splitted_name[0])

    ext_pos = import_name.find(".py")
    if ext_pos > 0:
      import_name = import_name[:ext_pos]
    
    mo = __import__(str(import_name))

    if pathChanged:
      sys.path = save_path

    file_path = file_path.replace("\\","/")
    file_path = file_path.replace("//","/")

    dll = self.DLLEntity(mo,OpenRTM_aist.Properties())
    dll.properties.setProperty("file_path",file_path)
    self._modules.registerObject(dll)
    

    if init_func is None:
      return file_name

    
    self.symbol(file_path,init_func)(self._mgr)

    return file_name


  ##
  # @if jp
  # @brief ⥸塼Υ
  #
  # ꤷɺѤߥ⥸塼򥯥ɤ롣
  #
  # @param self
  # @param file_name оݥ⥸塼̾
  #
  # @else
  # @brief Unload module
  # @endif
  def unload(self, file_name):
    dll = self._modules.find(file_name)
    if not dll:
      raise ModuleManager.NotFound(file_name)
    dll_name = dll.dll.__name__
    if dll_name in sys.modules.keys():
      sys.modules.pop(dll_name)
    self._modules.unregisterObject(file_name)
    return


  ##
  # @if jp
  # @brief ⥸塼Υ
  #
  # ƤΥɺѤߥ⥸塼򥢥ɤ롣
  #
  # @param self
  #
  # @else
  # @brief Unload all modules
  # @endif
  def unloadAll(self):
    dlls = self._modules.getObjects()
    
    for dll in dlls:
      ident = dll.properties.getProperty("file_path")
      self._modules.unregisterObject(ident)
    return


  ##
  # @if jp
  # @brief ⥸塼Υܥλ
  #
  # ⥸塼Υܥ
  #
  # @param self
  # @param file_name оݥե̾
  # @param func_name оݴؿ̾
  #
  # @else
  # @brief Look up a named symbol in the module
  # @endif
  def symbol(self, file_name, func_name):
    dll = self._modules.find(file_name)
    if not dll:
      raise ModuleManager.ModuleNotFound(file_name)

    func = getattr(dll.dll,func_name,None)

    if not func:
      raise ModuleManager.SymbolNotFound(func_name)
    
    return func


  ##
  # @if jp
  # @brief ⥸塼ɥѥꤹ
  # 
  # ⥸塼ɻоݥ⥸塼򸡺ѥꤹ롣
  #
  # @param self
  # @param load_path_list ⥸塼븡оݥѥꥹ
  #
  # @else
  # @brief Set default module load path
  # @endif
  def setLoadpath(self, load_path_list):
    self._loadPath = load_path_list
    return


  ##
  # @if jp
  # @brief ⥸塼ɥѥ
  # 
  # ꤵƤ⥸塼򸡺оݥѥꥹȤ롣
  #
  # @param self
  # 
  # @return load_path ⥸塼븡оݥѥꥹ
  #
  # @else
  # @brief Get default module load path
  # @endif
  def getLoadPath(self):
    return self._loadPath


  ##
  # @if jp
  # @brief ⥸塼ɥѥɲä
  # 
  # ꤵ줿ѥꥹȤ򸡺оݥѥꥹȤɲä롣
  #
  # @param self
  # @param load_path ɲå⥸塼븡оݥѥꥹ
  #
  # @else
  # @brief Add module load path
  # @endif
  def addLoadpath(self, load_path):
    for path in load_path:
      self._loadPath.append(path)
    return


  ##
  # @if jp
  # @brief ɺѤߤΥ⥸塼ꥹȤ
  #
  # ˥ɺѤߤΥ⥸塼ꥹȤ롣
  #
  # @param self
  #
  # @return ɺѤߥ⥸塼ꥹ
  #
  # @else
  # @brief Get loaded module names
  # @endif
  # std::vector<coil::Properties> getLoadedModules();
  def getLoadedModules(self):
    dlls = self._modules.getObjects()
    modules = []
    for dll in dlls:
      modules.append(dll.properties)

    return modules


  def __getRtcProfile(self, fname):
    # file name with full path
    fullname  = fname
    # directory name
    dirname   = os.path.dirname(fullname)
    sys.path.append(dirname)
    # basename
    basename  = os.path.basename(fullname)
    # classname
    classname  = basename.split(".")[0].lower()

    # loaded profile = old profiles - new profiles
    # for old
    oldp = self._mgr.getFactoryProfiles()

    # for new
    comp_spec_name = classname+"_spec"


    
        
    try:
      with open(str(fullname)) as f:
        if f.read().find(comp_spec_name) == -1:
          return None
      imp_file = __import__(basename.split(".")[0])
    except:
      return None
    comp_spec = getattr(imp_file,comp_spec_name,None)
    if not comp_spec:
      return None
    newp = OpenRTM_aist.Properties(defaults_str=comp_spec)

    profs = []
    
    exists = False
    for i in range(len(oldp)):
      if    oldp[i].getProperty("implementation_id") == newp.getProperty("implementation_id") and \
            oldp[i].getProperty("type_name") == newp.getProperty("type_name") and \
            oldp[i].getProperty("description") == newp.getProperty("description") and \
            oldp[i].getProperty("version") == newp.getProperty("version"):
        exists = True
    if not exists:
      profs.append(newp)

        
    # loaded component profile have to be one
    if len(profs) == 0:
      return OpenRTM_aist.Properties()

    #if len(profs) > 1:
    #  return None

    return profs[0]



  ##
  # @if jp
  # @brief ˤɥѥΥ֥ʥեꥹȤ֤
  #
  #
  # @param self
  # @param lang
  # @param modules
  #
  #
  # @else
  # @brief Getting loadable file list on the loadpath for given language
  # @endif
  def getModuleList(self, lang, modules):
    l = "manager.modules." + lang
    lprop = self._properties.getNode(l)

    paths = lprop.getProperty("load_paths").split(",")
    
    
    paths.extend(self._loadPath)
    paths = self.deleteSamePath(paths)
    
    

    suffixes = lprop.getProperty("suffixes").split(",")

      
    self._rtcout.RTC_DEBUG("suffixes: %s", OpenRTM_aist.flatten(suffixes))

    for path in paths:
      if not path:
        self._rtcout.RTC_WARN("Given load path is empty")
        continue
      self._rtcout.RTC_DEBUG("Module load path: %s", path)
      flist = []
      for suffix in suffixes:
        tmp = [suffix]
        OpenRTM_aist.eraseHeadBlank(tmp)
        suffix = tmp[0]
        
        tmp = []
        OpenRTM_aist.getFileList(path,suffix,tmp)
        
        #tmp = glob.glob(path + os.sep + '*.' + suffix)
        if lang == "Python":
          for f in tmp:
            if f.find("__init__.py") != -1:
              tmp.remove(f)

        self._rtcout.RTC_DEBUG("File list (path:%s, ext:%s): %s", (path,suffix,OpenRTM_aist.flatten(suffixes)))
        flist.extend(tmp)
      
      for f in flist:
        f = f.replace("\\","/")
        f = f.replace("//","/")
        self.addNewFile(f, modules)
    modules = list(set(modules))


        

  ##
  # @if jp
  # @brief å̵ѥmodulesɲä
  #
  #
  # @param self
  # @param fpath
  # @param modules
  #
  #
  # @else
  # @brief Adding file path not existing cache
  # @endif
  def addNewFile(self, fpath, modules):
    exists = False
    for modprof in self._modprofs:
      if modprof.getProperty("module_file_path") == fpath:
        exists = True
        self._rtcout.RTC_DEBUG("Module %s already exists in cache.",fpath)
        break
    for loadfailmod in self._loadfailmods:
      if loadfailmod == fpath:
        exists = True
        break
    if not exists:
      self._rtcout.RTC_DEBUG("New module: %s",fpath)
      modules.append(fpath)


  ##
  # @if jp
  # @brief 졢եꥹȤ⥸塼Υץѥƥ֤
  #
  #
  # @param self
  # @param lang
  # @param modules
  # @param modprops
  #
  #
  # @else
  # @brief Getting loadable file list on the loadpath for given language
  # @endif
  def getModuleProfiles(self, lang, modules, modprops):
    l = "manager.modules." + lang
    lprop = self._properties.getNode(l)

    paths = lprop.getProperty("load_paths").split(",")
    
    

    for mod_ in modules:
      if lang == "Python":
        prop = self.__getRtcProfile(mod_)
        if prop:
          prop.setProperty("module_file_name",os.path.basename(mod_))
          prop.setProperty("module_file_path", mod_)
          modprops.append(prop)
      else:
        prop = OpenRTM_aist.Properties()
        cmd = lprop.getProperty("profile_cmd")
        if os.name == "nt":
          cmd = "cmd /c " + cmd
        cmd = cmd + " \""+mod_+"\""
        
        try:
          ret = OpenRTM_aist.popen(cmd).split(os.linesep)
          count = 0
          for r in ret:
            pos = r.find(":")
            if r.find(":") != -1:
              count += 1
              key = r[0:pos]
              tmp = [key]
              OpenRTM_aist.eraseHeadBlank(tmp)
              key = tmp[0]
              
              value = r[pos+1:]
              tmp = [value]
              OpenRTM_aist.eraseHeadBlank(tmp)
              value = tmp[0]

              prop.setProperty(key, value)
          if count > 0:
            self._rtcout.RTC_DEBUG("rtcprof cmd sub process done.")
            prop.setProperty("module_file_name",os.path.basename(mod_))
            prop.setProperty("module_file_path", mod_)
            modprops.append(prop)
          else:
            self._loadfailmods.append(mod_)
              
          
        except:
          self._rtcout.RTC_ERROR("popen faild")
          self._loadfailmods.append(mod_)


  ##
  # @if jp
  # @brief ̵ʥ⥸塼ץե
  #
  #
  # @param self
  #
  #
  # @else
  # @brief Removing incalid module profiles
  # @endif
  def removeInvalidModules(self):
    for modprof in self._modprofs:
      if not os.path.isfile(modprof.getProperty("module_file_path")):
        self._modprofs.remove(modprof)
      

      
  ##
  # @if jp
  # @brief Ʊѥ
  #
  #
  # @param self
  # @param paths
  #
  #
  # @else
  # @brief 
  # @endif
  def deleteSamePath(self, paths):
    tmp_paths = []
    for path in paths:
      if path:
        abs_path = os.path.abspath(path).replace("\\","/")
        abs_tmp_paths = []
        for tmp_path in tmp_paths:
          abs_tmp_path = os.path.abspath(tmp_path).replace("\\","/")
          abs_tmp_paths.append(abs_tmp_path)
        if abs_path not in abs_tmp_paths:
          tmp_paths.append(path)
    return tmp_paths
      

  ##
  # @if jp
  # @brief ɲǽ⥸塼ꥹȤ(̤)
  #
  # ɲǽʥ⥸塼ΥꥹȤ롣
  #
  # @param self
  #
  # @return ɲǽ⥸塼ꥹ
  #
  # @else
  # @brief Get loadable module names
  # @endif
  def getLoadableModules(self):
    self._rtcout.RTC_TRACE("getLoadableModules()")
    # getting loadable module file path list.
    langs = self._properties.getProperty("manager.supported_languages").split(",")
    self._rtcout.RTC_DEBUG("langs: %s",self._properties.getProperty("manager.supported_languages"))

    for lang in langs:
      tmp = [lang]
      OpenRTM_aist.eraseHeadBlank(tmp)
      lang = tmp[0]
      
      modules_ = []
      self.getModuleList(lang, modules_)
      self._rtcout.RTC_DEBUG("%s: %s", (lang, OpenRTM_aist.flatten(modules_)))
      

      tmpprops = []
      self.getModuleProfiles(lang, modules_, tmpprops)
      self._rtcout.RTC_DEBUG("Modile profile size: %d (newly founded modules)",len(tmpprops))
      
      self._modprofs.extend(tmpprops)

    self._rtcout.RTC_DEBUG("Modile profile size: %d",len(self._modprofs))
    self.removeInvalidModules()
    self._rtcout.RTC_DEBUG("Modile profile size: %d (invalid mod-profiles deleted)",len(self._modprofs))

    return self._modprofs
    




  ##
  # @if jp
  # @brief ⥸塼Хѥ
  #
  # оݥ⥸塼ХѥĤ褦ꤹ롣
  #
  # @param self
  #
  # @else
  # @brief Allow absolute load path
  # @endif
  def allowAbsolutePath(self):
    self._absoluteAllowed = True


  ##
  # @if jp
  # @brief ⥸塼Хѥػ
  #
  # оݥ⥸塼Хѥػߤ褦ꤹ롣
  #
  # @param self
  #
  # @else
  # @brief Forbid absolute load path
  # @endif
  def disallowAbsolutePath(self):
    self._absoluteAllowed = False


  ##
  # @if jp
  # @brief ⥸塼URL
  #
  # оݥ⥸塼URLĤ롣
  # ꤬ĤƤ硢⥸塼ɤƥɤ뤳Ȥ
  # Ĥ롣
  #
  # @param self
  #
  # @else
  # @brief Allow module download
  # @endif
  def allowModuleDownload(self):
    self._downloadAllowed = True


  ##
  # @if jp
  # @brief ⥸塼URLػ
  #
  # оݥ⥸塼URLػߤ롣
  #
  # @param self
  #
  # @else
  # @brief Forbid module download
  # @endif
  def disallowModuleDownload(self):
    self._downloadAllowed = False


  ##
  # @if jp
  # @brief LoadPath Υեθ
  # 
  # ꤵ줿ѥˡꤵ줿ե뤬¸ߤ뤫ǧ롣
  #
  # @param self
  # @param fname оݥե̾
  # @param load_path ѥꥹ
  #
  # @return 줿ե̾
  #
  # @else
  # @brief Search file from load path
  # @endif
  def findFile(self, fname, load_path):
    file_name = fname
    for path in load_path:
      suffix = self._properties.getProperty("manager.modules.Python.suffixes")
      if fname.find("."+suffix) == -1:
        f = str(path) + os.sep + str(file_name)+"."+suffix
      else:
        f = str(path)+ os.sep + str(file_name)
      if self.fileExist(f):
        f = f.replace("\\","/")
        f = f.replace("//","/")
        return f
      filelist = []
      OpenRTM_aist.findFile(path,file_name,filelist)
      
      if len(filelist) > 0:
        return filelist[0]
    return ""


  ##
  # @if jp
  # @brief ե뤬¸ߤ뤫ɤΥå
  #
  # ꤵ줿ե뤬¸ߤ뤫ǧ롣
  #
  # @param self
  # @param filename ¸߳ǧоݥե̾
  #
  # @return ե¸߳ǧ(ե뤢:trueʤ:false)
  #
  # @else
  # @brief Check file existance
  # @endif
  def fileExist(self, filename):
    fname = filename
    suffix = self._properties.getProperty("manager.modules.Python.suffixes")
    if fname.find("."+suffix) == -1:
      fname = str(filename)+"."+suffix

    if os.path.isfile(fname):
      return True

    return False



  ##
  # @if jp
  # @brief ؿܥ
  #
  # ؿ̾ΤȤΩƤ롣
  #
  # @param self
  # @param file_path оݥ⥸塼̾
  #
  # @return ؿ̾ȤΩƷ
  #
  # @else
  # @brief Create initialize function symbol
  # @endif
  def getInitFuncName(self, file_path):
    base_name = os.path.basename(file_path)
    return str(self._initFuncPrefix)+str(base_name)+str(self._initFuncSuffix)



  ##
  # @if jp
  # @class DLL
  # @brief ⥸塼ݻ饹
  # @else
  #
  # @endif
  class DLL:
    def __init__(self, dll):
      self.dll = dll
      return


  class DLLEntity:
    def __init__(self,dll,prop):
      self.dll = dll
      self.properties = prop


  class DLLPred:
    def __init__(self, name=None, factory=None):
      self._filepath = name or factory

    def __call__(self, dll):
      return self._filepath == dll.properties.getProperty("file_path")
